#include <xc.h>     // XC8 compilateur
#include <stdint.h>
#include <stdio.h>     // uint8_t dfinitions
#include <stdlib.h>

#include "system.h"
#include "neopixel.h"
/**
 * @brief Impulsion de 800ns
 */
void One_Pulse(void)
{
    DIN = 1;        // environ 750ns = 6 x 125ns
    DIN = 1;
    DIN = 1;
    DIN = 1;
    DIN = 1;
    DIN = 1;
    DIN = 0;        // La dure du niveau bas est allonge par l'appel des autres fonctions
}
/**
 * @brief Impulsion de 400ns
 */
void Zero_Pulse(void)
{
    DIN = 1;        // environ 375ns = 3 x 125ns
    DIN = 1;
    DIN = 1;
    DIN = 0;
}
/**
 * @brief Envoi de 8 impulsions image de la luminosit
 * @param 
 */
// Fonction qui envoit la variable "valeur" de 8 bits
void NPX_Send_Byte(uint8_t valeur)
{
    if (valeur & 0b10000000)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0b01000000)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0x20)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0x10)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0x08)   One_Pulse();
    else    Zero_Pulse();
    if (valeur& 0x04)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0x02)   One_Pulse();
    else    Zero_Pulse();
    if (valeur & 0x01)   One_Pulse();
    else    Zero_Pulse();
}

#define NB_PIXELS 8

uint8_t neo_v[NB_PIXELS];
uint8_t neo_r[NB_PIXELS];
uint8_t neo_b[NB_PIXELS];

void NPX_Write_Pixel (uint8_t pos, uint8_t neo_vert, uint8_t neo_rouge, uint8_t neo_bleu)
{
    neo_v[pos] = neo_vert;
    neo_r[pos] = neo_rouge;
    neo_b[pos] = neo_bleu;
}

void Send_Array(void)
{
// Il est conseill de bloquer les interruptions pendant le transfert
// Cette partie est critique
    for (uint8_t i = 0; i < NB_PIXELS; i++)
    {
        NPX_Send_Byte(neo_v[i]);
        NPX_Send_Byte(neo_r[i]);
        NPX_Send_Byte(neo_b[i]);
    }
    __delay_us(50);         // Reset des nopixels
}

uint8_t pos, pos_old,etat_ch;
uint8_t coul_v, coul_r, coul_b;
uint16_t coul;

void Chenillard (void)
{
    pos_old = pos;
    if(!etat_ch)
    {
        pos++;
        if (pos == NB_PIXELS -1)
            etat_ch = 1;
    }
    else
    {
        pos--;
        if (pos == 0)
        {
            etat_ch = 0;
            coul = coul + 64;
            coul_v = coul & 0x00FF;
            coul_r = (coul & 0x0FF0)>>4;
            coul_b = (coul & 0xFF00)>>8;
        }
    }       
    NPX_Write_Pixel(pos_old,0,0,0);
    NPX_Write_Pixel(pos,coul_v,coul_r,coul_b);
    
    Send_Array();      
}